package com.jiuqi.util.inifile;

import java.io.OutputStream;
import java.io.InputStream;
import java.io.IOException;
import com.jiuqi.util.StreamException;
import com.jiuqi.util.MemStream;
import com.jiuqi.util.Stream;

public abstract class IniBuffer
{
    private int writeCount;
    private int readCount;
    
    public IniBuffer() {
        this.writeCount = 0;
        this.readCount = 0;
    }
    
    public abstract int getEntry();
    
    public abstract void writeEntry();
    
    public abstract boolean gotoEntry(final boolean p0);
    
    public abstract void gotoCursor();
    
    public abstract void writeInfoBoolean(final boolean p0);
    
    public abstract void writeInfoDouble(final double p0);
    
    public abstract void writeInfoInteger(final int p0);
    
    public abstract void writeInfoLong(final long p0);
    
    public abstract void writeInfoString(final String p0);
    
    public abstract void writeInfoByte(final byte p0);
    
    public abstract boolean readInfoBoolean();
    
    public abstract double readInfoDouble();
    
    public abstract int readInfoInteger();
    
    public abstract long readInfoLong();
    
    public abstract String readInfoString();
    
    public abstract byte readInfoByte();
    
    public abstract int writeBoolean(final boolean p0);
    
    public abstract int writeDouble(final double p0);
    
    public abstract int writeInteger(final int p0);
    
    public abstract int writeLong(final long p0);
    
    public abstract int writeString(final String p0);
    
    public abstract boolean readBoolean(final int p0);
    
    public abstract double readDouble(final int p0);
    
    public abstract int readInteger(final int p0);
    
    public abstract long readLong(final int p0);
    
    public abstract String readString(final int p0);
    
    public abstract int writeStream(final Stream p0, final int p1);
    
    public abstract void readStream(final Stream p0, final int p1, final int p2);
    
    public abstract void loadFromStream(final Stream p0);
    
    public abstract void replaceStream(final Stream p0);
    
    public abstract void saveToStream(final Stream p0);
    
    public final boolean verify() {
        return this.gotoEntry(true);
    }
    
    public void beginWrite() {
        if (this.writeCount == 0) {
            this.gotoCursor();
        }
        ++this.writeCount;
    }
    
    public boolean writeing() {
        return this.writeCount > 0;
    }
    
    public void endWrite() {
        --this.writeCount;
    }
    
    public void beginRead() {
        ++this.readCount;
    }
    
    public boolean reading() {
        return this.readCount > 0;
    }
    
    public void endRead() {
        --this.readCount;
    }
    
    public void loadFromFile(final String fileName) throws StreamException, IOException {
        final Stream stream = (Stream)new MemStream();
        stream.setPosition(0L);
        stream.loadFromFile(fileName);
        this.loadFromStream(stream);
    }
    
    public void saveToFile(final String fileName) throws StreamException, IOException {
        final Stream stream = (Stream)new MemStream();
        this.saveToStream(stream);
        stream.saveToFile(fileName);
    }
    
    public void loadFromStream(final InputStream inStream) throws StreamException, IOException {
        final Stream stream = (Stream)new MemStream();
        stream.setPosition(0L);
        stream.loadFromStream(inStream);
        this.loadFromStream(stream);
    }
    
    public void saveToStream(final OutputStream outStream) throws StreamException, IOException {
        final Stream stream = (Stream)new MemStream();
        this.saveToStream(stream);
        stream.saveToStream(outStream);
    }
}
